import logging
import os
from flask_wtf import FlaskForm
from flask_wtf.file import FileField, FileRequired, FileAllowed
from wtforms import SubmitField
from wtforms.validators import ValidationError
from flask import render_template, flash, redirect, url_for, request
from flask_login import login_user, logout_user, login_required, current_user
from werkzeug.utils import secure_filename
from app import app, db
from app.forms import LoginForm, RegistrationForm, UploadForm, ProfileForm
from app.models import User, File
from app.forms import RenameForm
from flask import send_from_directory
from datetime import datetime
from app.utils import add_login_attempt, check_and_ban_ip, is_ip_banned


logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)

ALLOWED_EXTENSIONS = {'rar','zip','txt', 'pdf', 'png', 'jpg', 'jpeg', 'gif', 'mp3', 'mp4', 'avi', 'mov', 'doc', 'docx'}

def validate_file_extension(form, field):
    if field.data:
        file_extension = field.data.filename.rsplit('.', 1)[1].lower()
        if file_extension not in ALLOWED_EXTENSIONS:
            raise ValidationError('File type not allowed. Allowed types: ' + ', '.join(ALLOWED_EXTENSIONS))

class UploadForm(FlaskForm):
    file = FileField('File', validators=[FileRequired(), validate_file_extension])
    submit = SubmitField('Upload')

@app.route('/')
@app.route('/index')
@login_required
def index():
    return redirect(url_for('dashboard'))

@app.route('/login', methods=['GET', 'POST'])
def login():
    if current_user.is_authenticated:
        return redirect(url_for('index'))
    
    ip_address = request.remote_addr
    if is_ip_banned(ip_address):
        flash('Too many failed login attempts. Please try again later.')
        return redirect(url_for('login'))
    
    form = LoginForm()
    if form.validate_on_submit():
        user = User.query.filter_by(username=form.username.data).first()
        if user is None or not user.check_password(form.password.data):
            add_login_attempt(ip_address, False)
            if check_and_ban_ip(ip_address):
                flash('Too many failed login attempts. Your IP has been temporarily banned.')
            else:
                flash('Invalid username or password')
            return redirect(url_for('login'))
        
        add_login_attempt(ip_address, True)
        login_user(user)
        return redirect(url_for('index'))
    
    return render_template('login.html', title='Sign In', form=form)

@app.route('/logout')
def logout():
    logout_user()
    return redirect(url_for('index'))

@app.route('/register', methods=['GET', 'PxOST'])
def register():
    if current_user.is_authenticated:
        return redirect(url_for('index'))
    form = RegistrationForm()
    if form.validate_on_submit():
        user = User(username=form.username.data)
        user.set_password(form.password.data)
        db.session.add(user)
        db.session.commit()
        flash('Congratulations, you are now a registered user!')
        return redirect(url_for('login'))
    return render_template('register.html', title='Register', form=form)

@app.route('/dashboard')
@login_required
def dashboard():
    files = current_user.files.all()
    return render_template('dashboard.html', title='Dashboard', files=files)

@app.route('/upload', methods=['GET', 'POST'])
@login_required
def upload():
    form = UploadForm()
    if form.validate_on_submit():
        uploaded_files = request.files.getlist('file')
        for file in uploaded_files:
            filename = secure_filename(file.filename)
            file.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))
            new_file = File(filename=filename, owner=current_user, date_uploaded=datetime.utcnow())
            db.session.add(new_file)
        db.session.commit()
        flash('Files uploaded successfully')
        return redirect(url_for('dashboard'))
    return render_template('upload.html', title='Upload Files', form=form)
@app.route('/delete/<int:file_id>')
@login_required
def delete(file_id):
    file = File.query.get_or_404(file_id)
    if file.owner != current_user:
        flash('You do not have permission to delete this file')
        return redirect(url_for('dashboard'))
    os.remove(os.path.join(app.config['UPLOAD_FOLDER'], file.filename))
    db.session.delete(file)
    db.session.commit()
    flash('File deleted successfully')
    return redirect(url_for('dashboard'))

@app.route('/rename/<int:file_id>', methods=['GET', 'POST'])
@login_required
def rename(file_id):
    file = File.query.get_or_404(file_id)
    if file.owner != current_user:
        flash('You do not have permission to rename this file')
        return redirect(url_for('dashboard'))
    form = RenameForm()
    if form.validate_on_submit():
        new_filename = form.new_filename.data
        old_extension = os.path.splitext(file.filename)[1]
        new_filename_with_extension = new_filename + old_extension
        if new_filename:
            old_path = os.path.join(app.config['UPLOAD_FOLDER'], file.filename)
            new_path = os.path.join(app.config['UPLOAD_FOLDER'], new_filename_with_extension)
            os.rename(old_path, new_path)
            file.filename = new_filename_with_extension
            db.session.commit()
            flash('File renamed successfully')
            return redirect(url_for('dashboard'))
    elif request.method == 'GET':
        form.new_filename.data = os.path.splitext(file.filename)[0]
    return render_template('rename.html', title='Rename File', form=form, file=file)


from flask import send_from_directory

@app.route('/download/<int:file_id>')
@login_required
def download(file_id):
    file = File.query.get_or_404(file_id)
    if file.owner != current_user:
        flash('You do not have permission to download this file')
        return redirect(url_for('dashboard'))
    return send_from_directory(app.config['UPLOAD_FOLDER'], file.filename, as_attachment=True)


@app.route('/profile', methods=['GET', 'POST'])
@login_required
def profile():
    form = ProfileForm()
    if form.validate_on_submit():
        current_user.username = form.username.data
        current_user.set_password(form.password.data)
        db.session.commit()
        flash('Your profile has been updated')
        return redirect(url_for('dashboard'))
    elif request.method == 'GET':
        form.username.data = current_user.username
    return render_template('profile.html', title='Profile', form=form)

